@@ -46,7 +46,7 @@ def mpf_norm(mpf, prec):
     if not man:
         # hack for mpf_normalize which does not do this;
         # it assumes that if man is zero the result is 0
-        # (see issue 3540)
+        # (see issue 6639)
         if not bc:
             return _mpf_zero
         else:
@@ -74,7 +74,7 @@ def seterr(divide=False):
 
 def _decimal_to_Rational_prec(dec):
     """Convert an ordinary decimal instance to a Rational."""
-    if not dec.is_finite():  # NOTE: this is_finite is not SymPy's
+    if not dec.is_finite():
         raise TypeError("dec must be finite, got %s." % dec)
     s, d, e = dec.as_tuple()
     prec = len(d)
@@ -216,14 +216,13 @@ class Number(AtomicExpr):
     """
     is_commutative = True
     is_number = True
+    is_Number = True
 
     __slots__ = []
 
     # Used to make max(x._prec, y._prec) return x._prec when only x is a float
     _prec = -1
 
-    is_Number = True
-
     def __new__(cls, *obj):
         if len(obj) == 1:
             obj = obj[0]
@@ -305,9 +304,6 @@ class Number(AtomicExpr):
             return -new
         return self  # there is no other possibility
 
-    def _eval_is_bounded(self):
-        return True
-
     def _eval_is_finite(self):
         return True
 
@@ -384,17 +380,33 @@ class Number(AtomicExpr):
             (self.__class__.__name__))
 
     def __lt__(self, other):
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s < %s" % (self, other))
         raise NotImplementedError('%s needs .__lt__() method' %
             (self.__class__.__name__))
 
     def __le__(self, other):
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s <= %s" % (self, other))
         raise NotImplementedError('%s needs .__le__() method' %
             (self.__class__.__name__))
 
     def __gt__(self, other):
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s > %s" % (self, other))
         return _sympify(other).__lt__(self)
 
     def __ge__(self, other):
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s >= %s" % (self, other))
         return _sympify(other).__le__(self)
 
     def __hash__(self):
@@ -580,7 +592,12 @@ class Float(Number):
     """
     __slots__ = ['_mpf_', '_prec']
 
-    is_rational = True
+    # A Float represents many real numbers,
+    # both rational and irrational.
+    is_rational = None
+    is_irrational = None
+    is_number = True
+
     is_real = True
 
     is_Float = True
@@ -706,15 +723,15 @@ class Float(Number):
     def _as_mpf_op(self, prec):
         return self._mpf_, max(prec, self._prec)
 
-    def _eval_is_bounded(self):
+    def _eval_is_finite(self):
         if self._mpf_ in (_mpf_inf, _mpf_ninf):
             return False
         return True
 
-    def _eval_is_finite(self):
-        if self._mpf_ in (_mpf_inf, _mpf_ninf, _mpf_zero):
-            return False
-        return True
+    def _eval_is_infinite(self):
+        if self._mpf_ in (_mpf_inf, _mpf_ninf):
+            return True
+        return False
 
     def _eval_is_integer(self):
         return self._mpf_ == _mpf_zero
@@ -874,52 +891,56 @@ class Float(Number):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other
+            raise TypeError("Invalid comparison %s > %s" % (self, other))
         if isinstance(other, NumberSymbol):
             return other.__le__(self)
         if other.is_comparable:
             other = other.evalf()
         if isinstance(other, Number):
-            return bool(mlib.mpf_gt(self._mpf_, other._as_mpf_val(self._prec)))
+            return _sympify(bool(
+                mlib.mpf_gt(self._mpf_, other._as_mpf_val(self._prec))))
         return Expr.__gt__(self, other)
 
     def __ge__(self, other):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other  -->  ! <=
+            raise TypeError("Invalid comparison %s >= %s" % (self, other))
         if isinstance(other, NumberSymbol):
             return other.__lt__(self)
         if other.is_comparable:
             other = other.evalf()
         if isinstance(other, Number):
-            return bool(mlib.mpf_ge(self._mpf_, other._as_mpf_val(self._prec)))
+            return _sympify(bool(
+                mlib.mpf_ge(self._mpf_, other._as_mpf_val(self._prec))))
         return Expr.__ge__(self, other)
 
     def __lt__(self, other):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other
+            raise TypeError("Invalid comparison %s < %s" % (self, other))
         if isinstance(other, NumberSymbol):
             return other.__ge__(self)
         if other.is_real and other.is_number:
             other = other.evalf()
         if isinstance(other, Number):
-            return bool(mlib.mpf_lt(self._mpf_, other._as_mpf_val(self._prec)))
+            return _sympify(bool(
+                mlib.mpf_lt(self._mpf_, other._as_mpf_val(self._prec))))
         return Expr.__lt__(self, other)
 
     def __le__(self, other):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other  -->  ! <=
+            raise TypeError("Invalid comparison %s <= %s" % (self, other))
         if isinstance(other, NumberSymbol):
             return other.__gt__(self)
         if other.is_real and other.is_number:
             other = other.evalf()
         if isinstance(other, Number):
-            return bool(mlib.mpf_le(self._mpf_, other._as_mpf_val(self._prec)))
+            return _sympify(bool(
+                mlib.mpf_le(self._mpf_, other._as_mpf_val(self._prec))))
         return Expr.__le__(self, other)
 
     def __hash__(self):
@@ -932,6 +953,10 @@ class Float(Number):
         import sage.all as sage
         return sage.RealNumber(str(self))
 
+    def __format__(self, format_spec):
+        return format(decimal.Decimal(str(self)), format_spec)
+
+
 # Add sympify converters
 converter[float] = converter[decimal.Decimal] = Float
 
@@ -1027,6 +1052,7 @@ class Rational(Number):
     is_real = True
     is_integer = False
     is_rational = True
+    is_number = True
 
     __slots__ = ['p', 'q']
 
@@ -1320,76 +1346,76 @@ class Rational(Number):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other  --> not <
+            raise TypeError("Invalid comparison %s > %s" % (self, other))
         if isinstance(other, NumberSymbol):
             return other.__le__(self)
-        if other.is_real and other.is_number and not isinstance(other, Rational):
-            other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Rational):
-                return bool(self.p*other.q > self.q*other.p)
+                return _sympify(bool(self.p*other.q > self.q*other.p))
             if isinstance(other, Float):
-                return bool(mlib.mpf_gt(
-                    self._as_mpf_val(other._prec), other._mpf_))
+                return _sympify(bool(mlib.mpf_gt(
+                    self._as_mpf_val(other._prec), other._mpf_)))
             if other is S.NaN:
                 return other.__le__(self)
+        elif other.is_number and other.is_real:
+            self, other = Integer(self.p), self.q*other
         return Expr.__gt__(self, other)
 
     def __ge__(self, other):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other  -->  not <=
+            raise TypeError("Invalid comparison %s >= %s" % (self, other))
         if isinstance(other, NumberSymbol):
             return other.__lt__(self)
-        if other.is_real and other.is_number and not isinstance(other, Rational):
-            other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Rational):
-                 return bool(self.p*other.q >= self.q*other.p)
+                 return _sympify(bool(self.p*other.q >= self.q*other.p))
             if isinstance(other, Float):
-                return bool(mlib.mpf_ge(
-                    self._as_mpf_val(other._prec), other._mpf_))
+                return _sympify(bool(mlib.mpf_ge(
+                    self._as_mpf_val(other._prec), other._mpf_)))
             if other is S.NaN:
                 return other.__lt__(self)
+        elif other.is_number and other.is_real:
+            self, other = Integer(self.p), self.q*other
         return Expr.__ge__(self, other)
 
     def __lt__(self, other):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other  --> not <
+            raise TypeError("Invalid comparison %s < %s" % (self, other))
         if isinstance(other, NumberSymbol):
             return other.__ge__(self)
-        if other.is_real and other.is_number and not isinstance(other, Rational):
-            other = other.evalf()
         if isinstance(other, Number):
             if isinstance(other, Rational):
-                return bool(self.p*other.q < self.q*other.p)
+                return _sympify(bool(self.p*other.q < self.q*other.p))
             if isinstance(other, Float):
-                return bool(mlib.mpf_lt(
-                    self._as_mpf_val(other._prec), other._mpf_))
+                return _sympify(bool(mlib.mpf_lt(
+                    self._as_mpf_val(other._prec), other._mpf_)))
             if other is S.NaN:
                 return other.__ge__(self)
+        elif other.is_number and other.is_real:
+            self, other = Integer(self.p), self.q*other
         return Expr.__lt__(self, other)
 
     def __le__(self, other):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other  -->  not <=
+            raise TypeError("Invalid comparison %s <= %s" % (self, other))
         if isinstance(other, NumberSymbol):
             return other.__gt__(self)
-        if other.is_real and other.is_number and not isinstance(other, Rational):
-            other = other.evalf()
-        if isinstance(other, Number):
+        elif isinstance(other, Number):
             if isinstance(other, Rational):
-                return bool(self.p*other.q <= self.q*other.p)
+                return _sympify(bool(self.p*other.q <= self.q*other.p))
             if isinstance(other, Float):
-                return bool(mlib.mpf_le(
-                    self._as_mpf_val(other._prec), other._mpf_))
+                return _sympify(bool(mlib.mpf_le(
+                    self._as_mpf_val(other._prec), other._mpf_)))
             if other is S.NaN:
                 return other.__gt__(self)
+        elif other.is_number and other.is_real:
+            self, other = Integer(self.p), self.q*other
         return Expr.__le__(self, other)
 
     def __hash__(self):
@@ -1539,6 +1565,7 @@ class Integer(Rational):
 
     q = 1
     is_integer = True
+    is_number = True
 
     is_Integer = True
 
@@ -1678,35 +1705,43 @@ class Integer(Rational):
         return not self.__eq__(other)
 
     def __gt__(self, other):
-        if isinstance(other, integer_types):
-            return (self.p > other)
-        elif isinstance(other, Integer):
-            return (self.p > other.p)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s > %s" % (self, other))
+        if isinstance(other, Integer):
+            return _sympify(self.p > other.p)
         return Rational.__gt__(self, other)
 
     def __lt__(self, other):
-        if isinstance(other, integer_types):
-            return (self.p < other)
-        elif isinstance(other, Integer):
-            return (self.p < other.p)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s < %s" % (self, other))
+        if isinstance(other, Integer):
+            return _sympify(self.p < other.p)
         return Rational.__lt__(self, other)
 
     def __ge__(self, other):
-        if isinstance(other, integer_types):
-            return (self.p >= other)
-        elif isinstance(other, Integer):
-            return (self.p >= other.p)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s >= %s" % (self, other))
+        if isinstance(other, Integer):
+            return _sympify(self.p >= other.p)
         return Rational.__ge__(self, other)
 
     def __le__(self, other):
-        if isinstance(other, integer_types):
-            return (self.p <= other)
-        elif isinstance(other, Integer):
-            return (self.p <= other.p)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s <= %s" % (self, other))
+        if isinstance(other, Integer):
+            return _sympify(self.p <= other.p)
         return Rational.__le__(self, other)
 
     def __hash__(self):
-        return super(Integer, self).__hash__()
+        return hash(self.p)
 
     def __index__(self):
         return self.p
@@ -1841,6 +1876,121 @@ for i_type in integer_types:
     converter[i_type] = Integer
 
 
+class AlgebraicNumber(Expr):
+    """Class for representing algebraic numbers in SymPy. """
+
+    __slots__ = ['rep', 'root', 'alias', 'minpoly']
+
+    is_AlgebraicNumber = True
+    is_algebraic = True
+    is_number = True
+
+    def __new__(cls, expr, coeffs=Tuple(), alias=None, **args):
+        """Construct a new algebraic number. """
+
+        from sympy.polys.polyclasses import ANP, DMP
+        from sympy.polys.numberfields import minimal_polynomial
+        from sympy.core.symbol import Symbol
+
+        expr = sympify(expr)
+
+        if isinstance(expr, (tuple, Tuple)):
+            minpoly, root = expr
+
+            if not minpoly.is_Poly:
+                minpoly = C.Poly(minpoly)
+        elif expr.is_AlgebraicNumber:
+            minpoly, root = expr.minpoly, expr.root
+        else:
+            minpoly, root = minimal_polynomial(
+                expr, args.get('gen'), polys=True), expr
+
+        dom = minpoly.get_domain()
+
+        if coeffs != Tuple():
+            if not isinstance(coeffs, ANP):
+                rep = DMP.from_sympy_list(sympify(coeffs), 0, dom)
+                scoeffs = Tuple(*coeffs)
+            else:
+                rep = DMP.from_list(coeffs.to_list(), 0, dom)
+                scoeffs = Tuple(*coeffs.to_list())
+
+            if rep.degree() >= minpoly.degree():
+                rep = rep.rem(minpoly.rep)
+
+            sargs = (root, scoeffs)
+
+        else:
+            rep = DMP.from_list([1, 0], 0, dom)
+
+            if root.is_negative:
+                rep = -rep
+
+            sargs = (root, coeffs)
+
+        if alias is not None:
+            if not isinstance(alias, Symbol):
+                alias = Symbol(alias)
+            sargs = sargs + (alias,)
+
+        obj = Expr.__new__(cls, *sargs)
+
+        obj.rep = rep
+        obj.root = root
+        obj.alias = alias
+        obj.minpoly = minpoly
+
+        return obj
+
+    def __hash__(self):
+        return super(AlgebraicNumber, self).__hash__()
+
+    def _eval_evalf(self, prec):
+        return self.as_expr()._evalf(prec)
+
+    @property
+    def is_aliased(self):
+        """Returns ``True`` if ``alias`` was set. """
+        return self.alias is not None
+
+    def as_poly(self, x=None):
+        """Create a Poly instance from ``self``. """
+        if x is not None:
+            return C.Poly.new(self.rep, x)
+        else:
+            if self.alias is not None:
+                return C.Poly.new(self.rep, self.alias)
+            else:
+                return C.PurePoly.new(self.rep, C.Dummy('x'))
+
+    def as_expr(self, x=None):
+        """Create a Basic expression from ``self``. """
+        return self.as_poly(x or self.root).as_expr().expand()
+
+    def coeffs(self):
+        """Returns all SymPy coefficients of an algebraic number. """
+        return [ self.rep.dom.to_sympy(c) for c in self.rep.all_coeffs() ]
+
+    def native_coeffs(self):
+        """Returns all native coefficients of an algebraic number. """
+        return self.rep.all_coeffs()
+
+    def to_algebraic_integer(self):
+        """Convert ``self`` to an algebraic integer. """
+        f = self.minpoly
+
+        if f.LC() == 1:
+            return self
+
+        coeff = f.LC()**(f.degree() - 1)
+        poly = f.compose(C.Poly(f.gen/f.LC()))
+
+        minpoly = poly*coeff
+        root = f.LC()*self.root
+
+        return AlgebraicNumber((minpoly, root), self.coeffs())
+
+
 class RationalConstant(Rational):
     """
     Abstract base class for rationals with specific behaviors
@@ -1885,9 +2035,9 @@ class Zero(with_metaclass(Singleton, IntegerConstant)):
     q = 1
     is_positive = False
     is_negative = False
-    is_finite = False
     is_zero = True
     is_composite = False
+    is_number = True
 
     __slots__ = []
 
@@ -1942,6 +2092,7 @@ class One(with_metaclass(Singleton, IntegerConstant)):
 
     .. [1] http://en.wikipedia.org/wiki/1_%28number%29
     """
+    is_number = True
 
     p = 1
     q = 1
@@ -1993,6 +2144,7 @@ class NegativeOne(with_metaclass(Singleton, IntegerConstant)):
     .. [1] http://en.wikipedia.org/wiki/%E2%88%921_%28number%29
 
     """
+    is_number = True
 
     p = -1
     q = 1
@@ -2047,6 +2199,7 @@ class Half(with_metaclass(Singleton, RationalConstant)):
 
     .. [1] http://en.wikipedia.org/wiki/One_half
     """
+    is_number = True
 
     p = 1
     q = 2
@@ -2098,12 +2251,8 @@ class Infinity(with_metaclass(Singleton, Number)):
 
     is_commutative = True
     is_positive = True
-    is_bounded = False
-    is_finite = False
-    is_infinitesimal = False
-    is_integer = None
-    is_rational = None
-    is_odd = None
+    is_infinite = True
+    is_number = True
 
     __slots__ = []
 
@@ -2236,29 +2385,59 @@ class Infinity(with_metaclass(Singleton, Number)):
     def __ne__(self, other):
         return other is not S.Infinity
 
-    @_sympifyit('other', NotImplemented)
     def __lt__(self, other):
-        if other.is_number and other.is_real is False:
-            raise TypeError("Invalid comparison of %s and %s" % (self, other))
-        return False
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s < %s" % (self, other))
+        if other.is_complex and other.is_real is False:
+            raise TypeError("Invalid complex comparison: %s and %s" %
+                            (self, other))
+        if other.is_real:
+            return S.false
+        return C.StrictLessThan(self, other, evaluate=False)
 
-    @_sympifyit('other', NotImplemented)
     def __le__(self, other):
-        if other.is_number and other.is_real is False:
-            raise TypeError("Invalid comparison of %s and %s" % (self, other))
-        return other is S.Infinity
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s <= %s" % (self, other))
+        if other.is_complex and other.is_real is False:
+            raise TypeError("Invalid complex comparison: %s and %s" %
+                            (self, other))
+        if other.is_real:
+            if other.is_finite or other is S.NegativeInfinity:
+                return S.false
+            elif other is S.Infinity:
+                return S.true
+        return C.LessThan(self, other, evaluate=False)
 
-    @_sympifyit('other', NotImplemented)
     def __gt__(self, other):
-        if other.is_number and other.is_real is False:
-            raise TypeError("Invalid comparison of %s and %s" % (self, other))
-        return other is not S.Infinity
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s > %s" % (self, other))
+        if other.is_complex and other.is_real is False:
+            raise TypeError("Invalid complex comparison: %s and %s" %
+                            (self, other))
+        if other.is_real:
+            if other.is_finite or other is S.NegativeInfinity:
+                return S.true
+            elif other is S.Infinity:
+                return S.false
+        return C.StrictGreaterThan(self, other, evaluate=False)
 
-    @_sympifyit('other', NotImplemented)
     def __ge__(self, other):
-        if other.is_number and other.is_real is False:
-            raise TypeError("Invalid comparison of %s and %s" % (self, other))
-        return True
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s >= %s" % (self, other))
+        if other.is_complex and other.is_real is False:
+            raise TypeError("Invalid complex comparison: %s and %s" %
+                            (self, other))
+        if other.is_real:
+            return S.true
+        return C.GreaterThan(self, other, evaluate=False)
 
     def __mod__(self, other):
         return S.NaN
@@ -2281,13 +2460,9 @@ class NegativeInfinity(with_metaclass(Singleton, Number)):
     """
 
     is_commutative = True
-    is_real = True
-    is_positive = False
-    is_bounded = False
-    is_finite = False
-    is_infinitesimal = False
-    is_integer = None
-    is_rational = None
+    is_negative = True
+    is_infinite = True
+    is_number = True
 
     __slots__ = []
 
@@ -2429,29 +2604,64 @@ class NegativeInfinity(with_metaclass(Singleton, Number)):
     def __ne__(self, other):
         return other is not S.NegativeInfinity
 
-    @_sympifyit('other', NotImplemented)
     def __lt__(self, other):
-        if other.is_number and other.is_real is False:
-            raise TypeError("Invalid comparison of %s and %s" % (self, other))
-        return other is not S.NegativeInfinity
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s < %s" % (self, other))
+        if other.is_complex and other.is_real is False:
+            raise TypeError("Invalid complex comparison: %s and %s" %
+                            (self, other))
+        if other.is_real:
+            if other.is_finite or other is S.Infinity:
+                return S.true
+            elif other is S.NegativeInfinity:
+                return S.false
+        return C.StrictLessThan(self, other, evaluate=False)
 
-    @_sympifyit('other', NotImplemented)
     def __le__(self, other):
-        if other.is_number and other.is_real is False:
-            raise TypeError("Invalid comparison of %s and %s" % (self, other))
-        return True
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s <= %s" % (self, other))
+        if other.is_complex and other.is_real is False:
+            raise TypeError("Invalid complex comparison: %s and %s" %
+                            (self, other))
+        if other.is_real:
+            return S.true
+        return C.LessThan(self, other, evaluate=False)
 
-    @_sympifyit('other', NotImplemented)
     def __gt__(self, other):
-        if other.is_number and other.is_real is False:
-            raise TypeError("Invalid comparison of %s and %s" % (self, other))
-        return False
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s > %s" % (self, other))
+        if other.is_complex and other.is_real is False:
+            raise TypeError("Invalid complex comparison: %s and %s" %
+                            (self, other))
+        if other.is_real:
+            return S.false
+        return C.StrictGreaterThan(self, other, evaluate=False)
 
-    @_sympifyit('other', NotImplemented)
     def __ge__(self, other):
-        if other.is_number and other.is_real is False:
-            raise TypeError("Invalid comparison of %s and %s" % (self, other))
-        return other is S.NegativeInfinity
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s >= %s" % (self, other))
+        if other.is_complex and other.is_real is False:
+            raise TypeError("Invalid complex comparison: %s and %s" %
+                            (self, other))
+        if other.is_real:
+            if other.is_finite or other is S.Infinity:
+                return S.false
+            elif other is S.NegativeInfinity:
+                return S.true
+        return C.GreaterThan(self, other, evaluate=False)
+
+    def __mod__(self, other):
+        return S.NaN
+
+    __rmod__ = __mod__
 
 
 class NaN(with_metaclass(Singleton, Number)):
@@ -2463,24 +2673,32 @@ class NaN(with_metaclass(Singleton, Number)):
     Python ``float('nan')``.
 
     NaN serves as a place holder for numeric values that are indeterminate.
-    Most operations on nan, produce another nan.  Most indeterminate forms,
-    such as ``0/0`` or ``oo - oo` produce nan.  Two exceptions are ``0**0``
+    Most operations on NaN, produce another NaN.  Most indeterminate forms,
+    such as ``0/0`` or ``oo - oo` produce NaN.  Two exceptions are ``0**0``
     and ``oo**0``, which all produce ``1`` (this is consistent with Python's
     float).
 
+    NaN is mathematically not equal to anything else, even NaN itself.  This
+    explains the initially counter-intuitive results with ``Eq`` and ``==`` in
+    the examples below.
+
     NaN is a singleton, and can be accessed by ``S.NaN``, or can be imported
     as ``nan``.
 
     Examples
     ========
 
-    >>> from sympy import nan, S, oo
+    >>> from sympy import nan, S, oo, Eq
     >>> nan is S.NaN
     True
     >>> oo - oo
     nan
     >>> nan + 1
     nan
+    >>> Eq(nan, nan)   # mathematical equality
+    False
+    >>> nan == nan     # structural equality
+    True
 
     References
     ==========
@@ -2491,14 +2709,16 @@ class NaN(with_metaclass(Singleton, Number)):
     is_commutative = True
     is_real = None
     is_rational = None
+    is_algebraic = None
+    is_transcendental = None
     is_integer = None
     is_comparable = False
     is_finite = None
-    is_bounded = None
     is_zero = None
     is_prime = None
     is_positive = None
     is_negative = None
+    is_number = True
 
     __slots__ = []
 
@@ -2537,22 +2757,43 @@ class NaN(with_metaclass(Singleton, Number)):
         return super(NaN, self).__hash__()
 
     def __eq__(self, other):
+        # NaN is structurally equal to another NaN
         return other is S.NaN
 
     def __ne__(self, other):
         return other is not S.NaN
 
+    def _eval_Eq(self, other):
+        # NaN is not mathematically equal to anything, even NaN
+        return S.false
+
     def __gt__(self, other):
-        return False
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s > %s" % (self, other))
+        return S.false
 
     def __ge__(self, other):
-        return False
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s >= %s" % (self, other))
+        return S.false
 
     def __lt__(self, other):
-        return False
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s < %s" % (self, other))
+        return S.false
 
     def __le__(self, other):
-        return False
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s <= %s" % (self, other))
+        return S.false
 
 nan = S.NaN
 
@@ -2587,8 +2828,7 @@ class ComplexInfinity(with_metaclass(Singleton, AtomicExpr)):
     """
 
     is_commutative = True
-    is_bounded = False
-    is_real = None
+    is_infinite = True
     is_number = True
 
     __slots__ = []
@@ -2626,7 +2866,6 @@ zoo = S.ComplexInfinity
 class NumberSymbol(AtomicExpr):
 
     is_commutative = True
-    is_bounded = True
     is_finite = True
     is_number = True
 
@@ -2665,41 +2904,57 @@ class NumberSymbol(AtomicExpr):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other  --> not <
+            raise TypeError("Invalid comparison %s < %s" % (self, other))
         if self is other:
-            return False
+            return S.false
         if isinstance(other, Number):
             approx = self.approximation_interval(other.__class__)
             if approx is not None:
                 l, u = approx
                 if other < l:
-                    return False
+                    return S.false
                 if other > u:
-                    return True
-            return self.evalf() < other
+                    return S.true
+            return _sympify(self.evalf() < other)
         if other.is_real and other.is_number:
             other = other.evalf()
-            return self.evalf() < other
+            return _sympify(self.evalf() < other)
         return Expr.__lt__(self, other)
 
     def __le__(self, other):
         try:
             other = _sympify(other)
         except SympifyError:
-            return False    # sympy > other  --> not <=
+            raise TypeError("Invalid comparison %s <= %s" % (self, other))
         if self is other:
-            return True
+            return S.true
         if other.is_real and other.is_number:
             other = other.evalf()
         if isinstance(other, Number):
-            return self.evalf() <= other
+            return _sympify(self.evalf() <= other)
         return Expr.__le__(self, other)
 
     def __gt__(self, other):
-        return (-self) < (-other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s > %s" % (self, other))
+        r = _sympify((-self) < (-other))
+        if r in (S.true, S.false):
+            return r
+        else:
+            return Expr.__gt__(self, other)
 
     def __ge__(self, other):
-        return (-self) <= (-other)
+        try:
+            other = _sympify(other)
+        except SympifyError:
+            raise TypeError("Invalid comparison %s >= %s" % (self, other))
+        r = _sympify((-self) <= (-other))
+        if r in (S.true, S.false):
+            return r
+        else:
+            return Expr.__ge__(self, other)
 
     def __int__(self):
         # subclass with appropriate return value
@@ -2741,6 +2996,9 @@ class Exp1(with_metaclass(Singleton, NumberSymbol)):
     is_positive = True
     is_negative = False  # XXX Forces is_negative/is_nonnegative
     is_irrational = True
+    is_number = True
+    is_algebraic = False
+    is_transcendental = True
 
     __slots__ = []
 
@@ -2817,6 +3075,9 @@ class Pi(with_metaclass(Singleton, NumberSymbol)):
     is_positive = True
     is_negative = False
     is_irrational = True
+    is_number = True
+    is_algebraic = False
+    is_transcendental = True
 
     __slots__ = []
 
@@ -2875,6 +3136,9 @@ class GoldenRatio(with_metaclass(Singleton, NumberSymbol)):
     is_positive = True
     is_negative = False
     is_irrational = True
+    is_number = True
+    is_algebraic = True
+    is_transcendental = False
 
     __slots__ = []
 
@@ -2937,6 +3201,7 @@ class EulerGamma(with_metaclass(Singleton, NumberSymbol)):
     is_positive = True
     is_negative = False
     is_irrational = None
+    is_number = True
 
     __slots__ = []
 
@@ -2992,6 +3257,7 @@ class Catalan(with_metaclass(Singleton, NumberSymbol)):
     is_positive = True
     is_negative = False
     is_irrational = None
+    is_number = True
 
     __slots__ = []
 
@@ -3040,9 +3306,10 @@ class ImaginaryUnit(with_metaclass(Singleton, AtomicExpr)):
 
     is_commutative = True
     is_imaginary = True
-    is_bounded = True
     is_finite = True
     is_number = True
+    is_algebraic = True
+    is_transcendental = False
 
     __slots__ = []
 
