@@ -10,12 +10,12 @@ from __future__ import print_function, division
 
 from sympy.core import S, C, sympify, pi, Dummy
 from sympy.core.logic import fuzzy_bool
-from sympy.core.numbers import oo, zoo
+from sympy.core.numbers import oo, zoo, Rational
 from sympy.simplify import simplify, trigsimp
 from sympy.functions.elementary.miscellaneous import sqrt, Max, Min
 from sympy.functions.elementary.complexes import im
 from sympy.geometry.exceptions import GeometryError
-from sympy.polys import Poly, PolynomialError
+from sympy.polys import Poly, PolynomialError, DomainError
 from sympy.solvers import solve
 from sympy.utilities.lambdify import lambdify
 from sympy.utilities.iterables import uniq
@@ -497,18 +497,27 @@ class Ellipse(GeometryEntity):
     def rotate(self, angle=0, pt=None):
         """Rotate ``angle`` radians counterclockwise about Point ``pt``.
 
-        Note: since the general ellipse is not supported, the axes of
-        the ellipse will not be rotated. Only the center is rotated to
-        a new position.
+        Note: since the general ellipse is not supported, only rotations that
+        are integer multiples of pi/2 are allowed.
 
         Examples
         ========
 
         >>> from sympy import Ellipse, pi
         >>> Ellipse((1, 0), 2, 1).rotate(pi/2)
-        Ellipse(Point(0, 1), 2, 1)
+        Ellipse(Point(0, 1), 1, 2)
+        >>> Ellipse((1, 0), 2, 1).rotate(pi)
+        Ellipse(Point(-1, 0), 2, 1)
         """
-        return super(Ellipse, self).rotate(angle, pt)
+        if self.hradius == self.vradius:
+            return self.func(*self.args)
+        if (angle/S.Pi).is_integer:
+            return super(Ellipse, self).rotate(angle, pt)
+        if (2*angle/S.Pi).is_integer:
+            return self.func(self.center.rotate(angle, pt), self.vradius, self.hradius)
+        # XXX see https://github.com/sympy/sympy/issues/2815 for general ellipes
+        raise NotImplementedError('Only rotations of pi/2 are currently supported for Ellipse.')
+
 
     def scale(self, x=1, y=1, pt=None):
         """Override GeometryEntity.scale since it is the major and minor
@@ -558,18 +567,7 @@ class Ellipse(GeometryEntity):
         zeros define the rotated ellipse is given.
 
         """
-        def _uniquely_named_symbol(xname, *exprs):
-            """Return a symbol which, when printed, will have a name unique
-            from any other already in the expressions given. The name is made
-            unique by prepending underscores.
-            """
-            prefix = '%s'
-            x = prefix % xname
-            syms = set.union(*[e.free_symbols for e in exprs])
-            while any(x == str(s) for s in syms):
-                prefix = '_' + prefix
-                x = prefix % xname
-            return _symbol(x)
+        from .util import _uniquely_named_symbol
 
         if line.slope in (0, oo):
             c = self.center
@@ -768,8 +766,8 @@ class Ellipse(GeometryEntity):
             inter = self.intersection(o)
             if isinstance(inter, Ellipse):
                 return False
-            return (inter is not None and isinstance(inter[0], Point)
-                    and len(inter) == 1)
+            return (inter is not None and len(inter) == 1
+                    and isinstance(inter[0], Point))
         elif isinstance(o, LinearEntity):
             inter = self._do_line_intersection(o)
             if inter is not None and len(inter) == 1:
@@ -850,20 +848,16 @@ class Ellipse(GeometryEntity):
             yis = solve(seq, y)[0]
             xeq = eq.subs(y, yis).as_numer_denom()[0].expand()
             try:
-                iv = list(zip(*Poly(xeq).intervals()))[0]
+                iv = list(zip(*Poly(xeq, x).intervals()))[0]
                 # bisection is safest here since other methods may miss root
                 xsol = [S(nroot(lambdify(x, xeq), i, solver="anderson"))
                     for i in iv]
                 points = [Point(i, solve(eq.subs(x, i), y)[0]).n(prec)
                     for i in xsol]
-            except PolynomialError:
-                pass
-        if not points:
-            points = [Point(*s) for s in solve((seq, eq), (x, y))]
-            # complicated expressions may not be decidably real so evaluate to
-            # check whether they are real or not
-            points = [i.n(prec) if prec is not None else i
-                for i in points if all(j.n(2).is_real for j in i.args)]
+            except (DomainError, PolynomialError):
+                xvals = solve(xeq, x)
+                points = [Point(xis, yis.xreplace({x: xis})) for xis in xvals]
+        points = [pt.n(prec) if prec is not None else pt for pt in points]
         slopes = [norm.subs(zip((x, y), pt.args)) for pt in points]
         if prec is not None:
             slopes = [i.n(prec) if i not in (-oo, oo, zoo) else i
@@ -1082,19 +1076,13 @@ class Ellipse(GeometryEntity):
         if det == 0:
             t = -b / a
             result.append(lp[0] + (lp[1] - lp[0]) * t)
-        else:
-            is_good = True
-            try:
-                is_good = (det > 0)
-            except NotImplementedError:  # symbolic, allow
-                is_good = True
-
-            if is_good:
-                root = sqrt(det)
-                t_a = (-b - root) / a
-                t_b = (-b + root) / a
-                result.append( lp[0] + (lp[1] - lp[0]) * t_a )
-                result.append( lp[0] + (lp[1] - lp[0]) * t_b )
+        # Definite and potential symbolic intersections are allowed.
+        elif (det > 0) != False:
+            root = sqrt(det)
+            t_a = (-b - root) / a
+            t_b = (-b + root) / a
+            result.append( lp[0] + (lp[1] - lp[0]) * t_a )
+            result.append( lp[0] + (lp[1] - lp[0]) * t_b )
 
         return [r for r in result if r in o]
 
@@ -1187,6 +1175,38 @@ class Ellipse(GeometryEntity):
 
         return o.intersection(self)
 
+    def evolute(self, x='x', y='y'):
+        """The equation of evolute of the ellipse.
+
+        Parameters
+        ==========
+
+        x : str, optional
+            Label for the x-axis. Default value is 'x'.
+        y : str, optional
+            Label for the y-axis. Default value is 'y'.
+
+        Returns
+        =======
+
+        equation : sympy expression
+
+        Examples
+        ========
+
+        >>> from sympy import Point, Ellipse
+        >>> e1 = Ellipse(Point(1, 0), 3, 2)
+        >>> e1.evolute()
+        2**(2/3)*y**(2/3) + (3*x - 3)**(2/3) - 5**(2/3)
+        """
+        if len(self.args) != 3:
+            raise NotImplementedError('Evolute of arbitrary Ellipse is not supported.')
+        x = _symbol(x)
+        y = _symbol(y)
+        t1 = (self.hradius*(x - self.center.x))**Rational(2, 3)
+        t2 = (self.vradius*(y - self.center.y))**Rational(2, 3)
+        return t1 + t2 - (self.hradius**2 - self.vradius**2)**Rational(2, 3)
+
     def __eq__(self, o):
         """Is the other GeometryEntity the same as this ellipse?"""
         return isinstance(o, GeometryEntity) and (self.center == o.center and
