@@ -25,8 +25,7 @@ def test_one_dof():
     forcing = KM.forcing
     rhs = MM.inv() * forcing
     assert expand(rhs[0]) == expand(-(q * k + u * c) / m)
-    assert KM.linearize() == \
-        (Matrix([[0, 1], [-k, -c]]), Matrix([]), Matrix([]))
+    assert KM.linearize(A_and_B=True, new_method=True)[0] == Matrix([[0, 1], [-k/m, -c/m]])
 
 
 def test_two_dof():
@@ -153,13 +152,9 @@ def test_rolling_disc():
     # This code tests our output vs. benchmark values. When r=g=m=1, the
     # critical speed (where all eigenvalues of the linearized equations are 0)
     # is 1 / sqrt(3) for the upright case.
-    forcing_full = KM.forcing_full
-    forcing_full.simplify()
-    rhs_full = KM.mass_matrix_full.inv() * forcing_full
-    rhs_jac = rhs_full.jacobian([q1, q2, q3, u1, u2, u3])
-    rhs_jac_subbed = rhs_jac.subs({r: 1, g: 1, m: 1})
-    rhs_jac_upright = rhs_jac_subbed.subs({q1: 0, q2: 0, q3: 0, u1: 0, u3: 0})
-    assert rhs_jac_upright.subs(u2, 1 / sqrt(3)).eigenvals() == {S(0): 6}
+    A = KM.linearize(A_and_B=True, new_method=True)[0]
+    A_upright = A.subs({r: 1, g: 1, m: 1}).subs({q1: 0, q2: 0, q3: 0, u1: 0, u3: 0})
+    assert A_upright.subs(u2, 1 / sqrt(3)).eigenvals() == {S(0): 6}
 
 
 def test_aux():
