@@ -209,7 +209,8 @@ class FracElement(DomainElement, DefaultPrinting, CantSympify):
         return f.raw_new(*numer.cancel(denom))
 
     def to_poly(f):
-        assert f.denom == 1
+        if f.denom != 1:
+            raise ValueError("f.denom should be 1")
         return f.numer
 
     def parent(self):
