@@ -8,10 +8,17 @@ from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.printing.pretty.pretty import pretty
 from sympy.tensor.tensor import TensorIndexType, tensor_indices, TensorSymmetry, \
     get_symmetric_group_sgs, TensorType, TensorIndex, tensor_mul, TensAdd, \
-    riemann_cyclic_replace, riemann_cyclic, TensMul, \
-    tensorsymmetry, tensorhead, TensorManager, TensExpr, TIDS
+    riemann_cyclic_replace, riemann_cyclic, TensMul, tensorsymmetry, tensorhead, \
+    TensorManager, TensExpr, TIDS
 from sympy.utilities.pytest import raises, skip
 
+def _is_equal(arg1, arg2):
+    if isinstance(arg1, TensExpr):
+        return arg1.equals(arg2)
+    elif isinstance(arg2, TensExpr):
+        return arg2.equals(arg1)
+    return arg1 == arg2
+
 
 #################### Tests from tensor_can.py #######################
 def test_canonicalize_no_slot_sym():
@@ -599,8 +606,8 @@ def test_special_eq_ne():
     p, q, r = tensorhead('p,q,r', [Lorentz], [[1]])
 
     t = 0*A(a, b)
-    assert t == 0
-    assert t == S.Zero
+    assert _is_equal(t, 0)
+    assert _is_equal(t, S.Zero)
 
     assert p(i) != A(a, b)
     assert A(a, -a) != A(a, b)
@@ -615,7 +622,7 @@ def test_special_eq_ne():
     assert p(i) - p(i) == 0
     assert p(i) - p(i) == S.Zero
 
-    assert A(a, b) == A(b, a)
+    assert _is_equal(A(a, b), A(b, a))
 
 def test_add2():
     Lorentz = TensorIndexType('Lorentz', dummy_fmt='L')
@@ -689,6 +696,24 @@ def test_substitute_indices():
     t1 = t.substitute_indices((-i, -j))
     assert t1 == p(j)
 
+    A_tmul = A(m, n)
+    A_c = A_tmul(m, -m)
+    assert _is_equal(A_c, A(n, -n))
+    ABm = A(i, j)*B(m, n)
+    ABc1 = ABm(i, j, -i, -j)
+    assert _is_equal(ABc1, A(i, -j)*B(-i, j))
+    ABc2 = ABm(i, -i, j, -j)
+    assert _is_equal(ABc2, A(m, -m)*B(-n, n))
+
+    asum = A(i, j) + B(i, j)
+    asc1 = asum(i, -i)
+    assert _is_equal(asc1, A(i, -i) + B(i, -i))
+
+    assert A(i, -i) == A(i, -i)()
+    assert A(i, -i) + B(-j, j) == ((A(i, -i) + B(i, -i)))()
+    assert _is_equal(A(i, j)*B(-j, k), (A(m, -j)*B(j, n))(i, k))
+    raises(ValueError, lambda: A(i, -i)(j, k))
+
 def test_riemann_cyclic_replace():
     Lorentz = TensorIndexType('Lorentz', dummy_fmt='L')
     m0, m1, m2, m3 = tensor_indices('m:4', Lorentz)
@@ -762,15 +787,15 @@ def test_contract_metric1():
     # g with both indices contracted with another tensor
     t1 = A(a,b)*B(-b,-c)*g(c, -a)
     t2 = t1.contract_metric(g)
-    assert t2 == A(a, b)*B(-b, -a)
+    assert _is_equal(t2, A(a, b)*B(-b, -a))
 
     t1 = A(a,b)*B(-b,-c)*g(c, d)*g(-a, -d)
     t2 = t1.contract_metric(g)
-    assert t2 == A(a,b)*B(-b,-a)
+    assert _is_equal(t2, A(a,b)*B(-b,-a))
 
     t1 = A(a,b)*g(-a,-b)
     t2 = t1.contract_metric(g)
-    assert t2 == A(a, -a)
+    assert _is_equal(t2, A(a, -a))
     assert not t2.free
     Lorentz = TensorIndexType('Lorentz', dummy_fmt='L')
     a, b = tensor_indices('a,b', Lorentz)
@@ -878,67 +903,67 @@ def test_metric_contract3():
 
     t = C(a0,a1)*B(-a1,-a0)
     t1 = t.contract_metric(C)
-    assert t1 == B(a0, -a0)
+    assert _is_equal(t1, B(a0, -a0))
 
     t = C(a1,a0)*B(-a1,-a0)
     t1 = t.contract_metric(C)
-    assert t1 == -B(a0, -a0)
+    assert _is_equal(t1, -B(a0, -a0))
 
     t = C(a0,-a1)*B(a1,-a0)
     t1 = t.contract_metric(C)
-    assert t1 == -B(a0, -a0)
+    assert _is_equal(t1, -B(a0, -a0))
 
     t = C(-a0,a1)*B(-a1,a0)
     t1 = t.contract_metric(C)
-    assert t1 == -B(a0, -a0)
+    assert _is_equal(t1, -B(a0, -a0))
 
     t = C(-a0,-a1)*B(a1,a0)
     t1 = t.contract_metric(C)
-    assert t1 == B(a0, -a0)
+    assert _is_equal(t1, B(a0, -a0))
 
     t = C(-a1, a0)*B(a1,-a0)
     t1 = t.contract_metric(C)
-    assert t1 == B(a0, -a0)
+    assert _is_equal(t1, B(a0, -a0))
 
     t = C(a0,a1)*psi(-a1)
     t1 = t.contract_metric(C)
-    assert t1 == psi(a0)
+    assert _is_equal(t1, psi(a0))
 
     t = C(a1,a0)*psi(-a1)
     t1 = t.contract_metric(C)
-    assert t1 == -psi(a0)
+    assert _is_equal(t1, -psi(a0))
 
     t = C(a0,a1)*chi(-a0)*psi(-a1)
     t1 = t.contract_metric(C)
-    assert t1 == -chi(a1)*psi(-a1)
+    assert _is_equal(t1, -chi(a1)*psi(-a1))
 
     t = C(a1,a0)*chi(-a0)*psi(-a1)
     t1 = t.contract_metric(C)
-    assert t1 == chi(a1)*psi(-a1)
+    assert _is_equal(t1, chi(a1)*psi(-a1))
 
     t = C(-a1,a0)*chi(-a0)*psi(a1)
     t1 = t.contract_metric(C)
-    assert t1 == chi(-a1)*psi(a1)
+    assert _is_equal(t1, chi(-a1)*psi(a1))
 
     t = C(a0, -a1)*chi(-a0)*psi(a1)
     t1 = t.contract_metric(C)
-    assert t1 == -chi(-a1)*psi(a1)
+    assert _is_equal(t1, -chi(-a1)*psi(a1))
 
     t = C(-a0,-a1)*chi(a0)*psi(a1)
     t1 = t.contract_metric(C)
-    assert t1 == chi(-a1)*psi(a1)
+    assert _is_equal(t1, chi(-a1)*psi(a1))
 
     t = C(-a1,-a0)*chi(a0)*psi(a1)
     t1 = t.contract_metric(C)
-    assert t1 == -chi(-a1)*psi(a1)
+    assert _is_equal(t1, -chi(-a1)*psi(a1))
 
     t = C(-a1,-a0)*B(a0,a2)*psi(a1)
     t1 = t.contract_metric(C)
-    assert t1 == -B(-a1,a2)*psi(a1)
+    assert _is_equal(t1, -B(-a1,a2)*psi(a1))
 
     t = C(a1,a0)*B(-a2,-a0)*psi(-a1)
     t1 = t.contract_metric(C)
-    assert t1 == B(-a2,a1)*psi(-a1)
+    assert _is_equal(t1, B(-a2,a1)*psi(-a1))
 
 def test_epsilon():
     Lorentz = TensorIndexType('Lorentz', dim=4, dummy_fmt='L')
@@ -1067,8 +1092,8 @@ def test_TensorManager():
     assert t1 == ps*ps + 2*ps*psh + psh*psh
     qs = G(i)*q(-i)
     qsh = GH(ih)*qh(-ih)
-    assert ps*qsh == qsh*ps
-    assert ps*qs != qs*ps
+    assert _is_equal(ps*qsh, qsh*ps)
+    assert not _is_equal(ps*qs, qs*ps)
     n = TensorManager.comm_symbols2i(Gsymbol)
     assert TensorManager.comm_i2symbol(n) == Gsymbol
 
@@ -1161,36 +1186,36 @@ def test_hidden_indices_for_matrix_multiplication():
     B0 = B(-m0)
     B1 = B(m1)
 
-    assert (B1*A0*B0) == B(m1, s0)*A(m0, -s0, s1)*B(-m0, -s1)
-    assert (B0*A0) == B(-m0, s0)*A(m0, -s0, -S.auto_right)
-    assert (A0*B0) == A(m0, S.auto_left, s0)*B(-m0, -s0)
+    assert _is_equal((B1*A0*B0), B(m1, s0)*A(m0, -s0, s1)*B(-m0, -s1))
+    assert _is_equal((B0*A0), B(-m0, s0)*A(m0, -s0, -S.auto_right))
+    assert _is_equal((A0*B0), A(m0, S.auto_left, s0)*B(-m0, -s0))
 
     C = tensorhead('C', [L, L], [[1]*2])
 
-    assert (C(True, True)) == C(L.auto_left, -L.auto_right)
+    assert _is_equal((C(True, True)), C(L.auto_left, -L.auto_right))
 
-    assert (A(m0)*C(m1, -m0)) == A(m2, S.auto_left, -S.auto_right)*C(m1, -m2)
+    assert _is_equal((A(m0)*C(m1, -m0)), A(m2, S.auto_left, -S.auto_right)*C(m1, -m2))
 
-    assert (C(True, True)*C(True, True)) == C(L.auto_left, m0)*C(-m0, -L.auto_right)
+    assert _is_equal((C(True, True)*C(True, True)), C(L.auto_left, m0)*C(-m0, -L.auto_right))
 
-    assert A(m0) == A(m0)
-    assert B(-m1) == B(-m1)
+    assert _is_equal(A(m0), A(m0))
+    assert _is_equal(B(-m1), B(-m1))
 
-    assert A(m0) - A(m0) == 0
+    assert _is_equal(A(m0) - A(m0), 0)
     ts1 = A(m0)*A(m1) + A(m1)*A(m0)
     ts2 = A(m1)*A(m0) + A(m0)*A(m1)
-    assert ts1 == ts2
-    assert A(m0)*A(m1) + A(m1)*A(m0) == A(m1)*A(m0) + A(m0)*A(m1)
+    assert _is_equal(ts1, ts2)
+    assert _is_equal(A(m0)*A(m1) + A(m1)*A(m0), A(m1)*A(m0) + A(m0)*A(m1))
 
-    assert A(m0) == (2*A(m0))/2
-    assert A(m0) == -(-A(m0))
-    assert 2*A(m0) - 3*A(m0) == -A(m0)
-    assert 2*D(m0, m1) - 5*D(m1, m0) == -3*D(m0, m1)
+    assert _is_equal(A(m0), (2*A(m0))/2)
+    assert _is_equal(A(m0), -(-A(m0)))
+    assert _is_equal(2*A(m0) - 3*A(m0), -A(m0))
+    assert _is_equal(2*D(m0, m1) - 5*D(m1, m0), -3*D(m0, m1))
 
     D0 = D(True, True, True, True)
     Aa = A(True, True, True)
 
-    assert D0 * Aa == D(L.auto_left, m0, S.auto_left, s0)*A(-m0, -s0, -S.auto_right)
+    assert _is_equal(D0 * Aa, D(L.auto_left, m0, S.auto_left, s0)*A(-m0, -s0, -S.auto_right))
     assert D(m0, m1) == D(m0, m1, S.auto_left, -S.auto_right)
 
     raises(ValueError, lambda: C(True))
@@ -1218,7 +1243,10 @@ def test_hidden_indices_for_matrix_multiplication():
 
 numpy = import_module('numpy')
 
-if numpy:
+def _get_valued_base_test_variables():
+    if numpy is None:
+        return
+
     minkowski = Matrix((
         (1, 0, 0, 0),
         (0, -1, 0, 0),
@@ -1259,7 +1287,7 @@ if numpy:
 
     ### non-diagonal metric ###
     ndm_matrix = (
-        (0, 1, 0,),
+        (1, 1, 0,),
         (1, 0, 1),
         (0, 1, 0,),
     )
@@ -1273,10 +1301,17 @@ if numpy:
     NC = tensorhead('NC', [ndm]*3, [[1]]*3)
     NC.data = [[[i+j+k for k in range(4, 7)] for j in range(1, 4)] for i in range(2, 5)]
 
+    return (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+            n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4)
+
 def test_valued_tensor_iter():
     numpy = import_module("numpy")
     if numpy is None:
         skip("numpy not installed.")
+
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
     # iteration on VTensorHead
     assert list(A) == [E, px, py, pz]
     assert list(ba_matrix) == list(BA)
@@ -1299,6 +1334,9 @@ def test_valued_tensor_covariant_contravariant_elements():
     if numpy is None:
         skip("numpy not installed.")
 
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
     assert A(-i0)[0] == A(i0)[0]
     assert A(-i0)[1] == -A(i0)[1]
 
@@ -1313,6 +1351,9 @@ def test_valued_tensor_get_matrix():
     if numpy is None:
         skip("numpy not installed.")
 
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
     matab = AB(i0, i1).get_matrix()
     assert matab == Matrix([
                             [1,  0,  0,  0],
@@ -1334,17 +1375,20 @@ def test_valued_tensor_contraction():
     if numpy is None:
         skip("numpy not installed.")
 
-    assert A(i0) * A(-i0) == E ** 2 - px ** 2 - py ** 2 - pz ** 2
-    assert A(i0) * A(-i0) == A ** 2
-    assert A(i0) * A(-i0) == A(i0) ** 2
-    assert (A(i0) * B(-i0)) == -px - 2 * py - 3 * pz
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
+    assert (A(i0) * A(-i0)).data == E ** 2 - px ** 2 - py ** 2 - pz ** 2
+    assert (A(i0) * A(-i0)).data == A ** 2
+    assert (A(i0) * A(-i0)).data == A(i0) ** 2
+    assert (A(i0) * B(-i0)).data == -px - 2 * py - 3 * pz
 
     for i in range(4):
         for j in range(4):
             assert (A(i0) * B(-i1))[i, j] == [E, px, py, pz][i] * [0, -1, -2, -3][j]
 
     # test contraction on the alternative Minkowski metric: [-1, 1, 1, 1]
-    assert C(mu0) * C(-mu0) == -E ** 2 + px ** 2 + py ** 2 + pz ** 2
+    assert (C(mu0) * C(-mu0)).data == -E ** 2 + px ** 2 + py ** 2 + pz ** 2
 
     contrexp = A(i0) * AB(i1, -i0)
     assert A(i0).rank == 1
@@ -1359,8 +1403,11 @@ def test_valued_tensor_self_contraction():
     if numpy is None:
         skip("numpy not installed.")
 
-    assert AB(i0, -i0) == 4
-    assert BA(i0, -i0) == 2
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
+    assert AB(i0, -i0).data == 4
+    assert BA(i0, -i0).data == 2
 
 
 def test_valued_tensor_pow():
@@ -1368,6 +1415,9 @@ def test_valued_tensor_pow():
     if numpy is None:
         skip("numpy not installed.")
 
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
     assert C**2 == -E**2 + px**2 + py**2 + pz**2
     assert C**1 == sqrt(-E**2 + px**2 + py**2 + pz**2)
     assert C(mu0)**2 == C**2
@@ -1379,13 +1429,16 @@ def test_valued_tensor_expressions():
     if numpy is None:
         skip("numpy not installed.")
 
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
     x1, x2, x3 = symbols('x1:4')
 
     # test coefficient in contraction:
     rank2coeff = x1 * A(i3) * B(i2)
     assert rank2coeff[1, 1] == x1 * px
     assert rank2coeff[3, 3] == 3 * pz * x1
-    coeff_expr = (x1 * A(i4)) * (B(-i4) / x2)
+    coeff_expr = ((x1 * A(i4)) * (B(-i4) / x2)).data
 
     assert coeff_expr.expand() == -px*x1/x2 - 2*py*x1/x2 - 3*pz*x1/x2
 
@@ -1403,16 +1456,16 @@ def test_valued_tensor_expressions():
     assert sub_expr[2] == py - 2
     assert sub_expr[3] == pz - 3
 
-    assert add_expr * B(-i0) == -px - 2*py - 3*pz - 14
+    assert (add_expr * B(-i0)).data == -px - 2*py - 3*pz - 14
 
     expr1 = x1*A(i0) + x2*B(i0)
     expr2 = expr1 * B(i1) * (-4)
     expr3 = expr2 + 3*x3*AB(i0, i1)
     expr4 = expr3 / 2
     assert expr4 * 2 == expr3
-    expr5 = expr4 * BA(-i1, -i0)
+    expr5 = (expr4 * BA(-i1, -i0))
 
-    assert expr5 == 28*E*x1 + 12*px*x1 + 20*py*x1 + 28*pz*x1 + 136*x2 + 3*x3
+    assert expr5.data.expand() == 28*E*x1 + 12*px*x1 + 20*py*x1 + 28*pz*x1 + 136*x2 + 3*x3
 
 
 def test_noncommuting_components():
@@ -1420,6 +1473,9 @@ def test_noncommuting_components():
     if numpy is None:
         skip("numpy not installed.")
 
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
     euclid = TensorIndexType('Euclidean')
     euclid.data = [1, 1]
     i1, i2, i3 = tensor_indices('i1:4', euclid)
@@ -1432,10 +1488,10 @@ def test_noncommuting_components():
 
     vtp = V1(i1, i2) * V2(-i2, -i1)
 
-    assert vtp == a**2 + b**2 + c**2 + d**2
-    assert vtp != a**2 + 2*b*c + d**2
+    assert vtp.data == a**2 + b**2 + c**2 + d**2
+    assert vtp.data != a**2 + 2*b*c + d**2
 
-    Vc = b * V1(i1, -i1)
+    Vc = (b * V1(i1, -i1)).data
     assert Vc.expand() == b * a + b * d
 
 
@@ -1444,64 +1500,95 @@ def test_valued_non_diagonal_metric():
     if numpy is None:
         skip("numpy not installed.")
 
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
     mmatrix = Matrix(ndm_matrix)
-    assert NA(n0)*NA(-n0) == (NA(n0).get_matrix().T * mmatrix * NA(n0).get_matrix())[0, 0]
+    assert (NA(n0)*NA(-n0)).data == (NA(n0).get_matrix().T * mmatrix * NA(n0).get_matrix())[0, 0]
 
 
-def test_valued_tensor_strip():
+def test_valued_assign_numpy_ndarray():
     numpy = import_module("numpy")
     if numpy is None:
         skip("numpy not installed.")
 
-    sA = A(i0).strip()
-    sB = B(-i0).strip()
-    sAB = AB(i0, i1).strip()
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
 
-    assert sA.data is None
-    assert sB.data is None
-    assert sAB.data is None
+    # this is needed to make sure that a numpy.ndarray can be assigned to a
+    # tensor.
+    arr = [E+1, px-1, py, pz]
+    A.data = numpy.array(arr)
+    for i in range(4):
+            assert A(i0).data[i] == arr[i]
 
-    sthA = A.strip()
-    sthB = B.strip()
-    sthAB = AB.strip()
+    qx, qy, qz = symbols('qx qy qz')
+    A(-i0).data = numpy.array([E, qx, qy, qz])
+    for i in range(4):
+        assert A(i0).data[i] == [E, -qx, -qy, -qz][i]
+        assert A.data[i] == [E, -qx, -qy, -qz][i]
 
-    assert sA.data is None
-    assert sB.data is None
-    assert sAB.data is None
+    # test on multi-indexed tensors.
+    random_4x4_data = [[(i**3-3*i**2)%(j+7) for i in range(4)] for j in range(4)]
+    AB(-i0, -i1).data = random_4x4_data
+    for i in range(4):
+        for j in range(4):
+            assert AB(i0, i1).data[i, j] == random_4x4_data[i][j]*(-1 if i else 1)*(-1 if j else 1)
+            assert AB(-i0, i1).data[i, j] == random_4x4_data[i][j]*(-1 if j else 1)
+            assert AB(i0, -i1).data[i, j] == random_4x4_data[i][j]*(-1 if i else 1)
+            assert AB(-i0, -i1).data[i, j] == random_4x4_data[i][j]
 
+    AB(-i0, i1).data = random_4x4_data
+    for i in range(4):
+        for j in range(4):
+            assert AB(i0, i1).data[i, j] == random_4x4_data[i][j]*(-1 if i else 1)
+            assert AB(-i0, i1).data[i, j] == random_4x4_data[i][j]
+            assert AB(i0, -i1).data[i, j] == random_4x4_data[i][j]*(-1 if i else 1)*(-1 if j else 1)
+            assert AB(-i0, -i1).data[i, j] == random_4x4_data[i][j]*(-1 if j else 1)
 
-def test_valued_tensor_applyfunc():
+def test_valued_metric_inverse():
     numpy = import_module("numpy")
     if numpy is None:
-        skip("numpy not installed.")
+        return
 
-    aA = A(i0).applyfunc(lambda x: x**2)
-    aB = B(i0).applyfunc(lambda x: x**3)
-    aB2 = B(-i0).applyfunc(lambda x: x**3)
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
+    # let's assign some fancy matrix, just to verify it:
+    # (this has no physical sense, it's just testing sympy);
+    # it is symmetrical:
+    md = [[2, 2, 2, 1], [2, 3, 1, 0], [2, 1, 2, 3], [1, 0, 3, 2]]
+    Lorentz.data = md
+    m = Matrix(md)
+    metric = Lorentz.metric
+    minv = m.inv()
+
+    meye = eye(4)
+
+    # the Kronecker Delta:
+    KD = Lorentz.get_kronecker_delta()
 
     for i in range(4):
-        assert aA[i] == A(i0)[i]**2
-        assert aB[i] == B(i1)[i]**3
-    assert aB*aB2 == -794
+        for j in range(4):
+            assert metric(i0, i1).data[i, j] == m[i, j]
+            assert metric(-i0, -i1).data[i, j] == minv[i, j]
+            assert metric(i0, -i1).data[i, j] == meye[i, j]
+            assert metric(-i0, i1).data[i, j] == meye[i, j]
+            assert metric(i0, i1)[i, j] == m[i, j]
+            assert metric(-i0, -i1)[i, j] == minv[i, j]
+            assert metric(i0, -i1)[i, j] == meye[i, j]
+            assert metric(-i0, i1)[i, j] == meye[i, j]
 
-    tA = A.applyfunc(lambda x: x + 33)
-    tB = B.applyfunc(lambda x: x + 33)
-    tAB = AB.applyfunc(lambda x: x + 33)
-
-    assert (tA(i0)*tA(-i0)).expand() == ((E + 33)**2 - (px + 33)**2 - (py + 33)**2 - (pz + 33)**2).expand()
-    assert tB(i0).get_matrix() == Matrix([33, 34, 35, 36])
-    assert tAB(i0, i1).get_matrix() == Matrix([
-        [34, 33, 33, 33],
-        [33, 32, 33, 33],
-        [33, 33, 32, 33],
-        [33, 33, 33, 32],
-    ])
+            assert KD(i0, -i1)[i, j] == meye[i, j]
 
 def test_valued_canon_bp_swapaxes():
     numpy = import_module("numpy")
     if numpy is None:
         return
 
+    (A, B, AB, BA, C, Lorentz, E, px, py, pz, LorentzD, mu0, mu1, mu2, ndm, n0, n1,
+     n2, NA, NB, NC, minkowski, ba_matrix, ndm_matrix, i0, i1, i2, i3, i4) = _get_valued_base_test_variables()
+
     e1 = A(i1)*A(i0)
     e1.data[0, 1] = 44
     e2 = e1.canon_bp()
